@@ -28,6 +28,8 @@
 #include <libecal/e-cal-types.h>
 #include <libecal/e-cal.h>
 #include <libecal/e-cal-time-util.h>
+#include <libedataserver/e-url.h>
+#include <libedataserverui/e-passwords.h>
 #include <pi-source.h>
 #include <pi-socket.h>
 #include <pi-dlp.h>
@@ -148,22 +150,29 @@ calconduit_load_configuration (guint32 pilot_id)
 	c->secret = gnome_config_get_bool ("secret=FALSE");
 	c->multi_day_split = gnome_config_get_bool ("multi_day_split=TRUE");
 	if ((c->last_uri = gnome_config_get_string ("last_uri")) && !strncmp (c->last_uri, "file://", 7)) {
-		const char *path = c->last_uri + 7;
+		char *filename = g_filename_from_uri (c->last_uri, NULL, NULL);
+		const char *path = filename;
 		const char *home;
 		
 		home = g_get_home_dir ();
 		
 		if (!strncmp (path, home, strlen (home))) {
 			path += strlen (home);
-			if (*path == '/')
+			if (G_IS_DIR_SEPARATOR (*path))
 				path++;
 			
 			if (!strcmp (path, "evolution/local/Calendar/calendar.ics")) {
+				gchar *new_filename =
+					g_build_filename (home,
+							  ".evolution/calendar/local/system/calendar.ics",
+							  NULL);
 				/* need to upgrade the last_uri. yay. */
 				g_free (c->last_uri);
-				c->last_uri = g_strdup_printf ("file://%s/.evolution/calendar/local/system/calendar.ics", home);
+				c->last_uri = g_filename_to_uri (new_filename, NULL, NULL);
+				g_free (new_filename);
 			}
 		}
+		g_free (filename);
 	}
 	
 	gnome_config_pop_prefix (); 
@@ -426,21 +435,61 @@ static char *print_remote (GnomePilotRecord *remote)
 	return buff;
 }
 
+static char *
+auth_func_cb (ECal *ecal, const char* prompt, const char *key, gpointer user_data)
+{
+	char *password;
+	ESource *source;
+	const gchar *auth_domain, *component_name;
+
+	source = e_cal_get_source (ecal);
+	auth_domain = e_source_get_property (source, "auth-domain");
+	component_name = auth_domain ? auth_domain : "Calendar";
+	password = e_passwords_get_password (component_name, key);
+
+	return password;
+}
+
 static int
 start_calendar_server (ECalConduitContext *ctxt)
 {
+	char *str_uri = NULL;
+	char *pass_key = NULL;
+	int retval = 0;
+
 	g_return_val_if_fail (ctxt != NULL, -2);
 
 	if (ctxt->cfg->source) {
 		ctxt->client = e_cal_new (ctxt->cfg->source, E_CAL_SOURCE_TYPE_EVENT);
+		/* Set the default timezone on the backend.
+		   As of Evo. 2.5.x, timezone should be set before
+		   calling e_cal_open.
+		*/
+	
+		if (ctxt->timezone) {
+			if (!e_cal_set_default_timezone (ctxt->client, ctxt->timezone, NULL))
+				return -1;
+		}
+		
+		if (e_source_get_property (ctxt->cfg->source, "auth")) {
+			EUri *e_uri;
+
+			LOG (g_message ("Authenticating calendar\n"));
+			str_uri = e_source_get_uri (ctxt->cfg->source);
+			e_uri = e_uri_new (str_uri);
+			pass_key = e_uri_to_string (e_uri, FALSE);
+			e_uri_free (e_uri);
+			if (ctxt->client)
+				e_cal_set_auth_func (ctxt->client, (ECalAuthFunc) auth_func_cb, NULL);
+		}
 		if (!e_cal_open (ctxt->client, TRUE, NULL))
-			return -1;
+			retval = -1;
 	} else if (!e_cal_open_default (&ctxt->client, E_CAL_SOURCE_TYPE_EVENT, NULL, NULL, NULL)) {
-		return -1;
+		retval = -1;
 	}
-
-        return 0;
-
+	g_free (str_uri);
+	g_free (pass_key);
+	return retval;
 }
 
 /* Utility routines */
@@ -616,9 +665,10 @@ process_multi_day (ECalConduitContext *ctxt, ECalChange *ccc, GList **multi_comp
  	old_end_value = dt_end.value;
 	while (!last) {
 		ECalComponent *clone = e_cal_component_clone (ccc->comp);
+		icalcomponent *ical_comp = NULL;
 		char *new_uid = e_cal_component_gen_uid ();
 		struct icaltimetype start_value, end_value;
-		ECalChange *c = g_new0 (ECalChange, 1);
+		ECalChange *c = NULL;
 		
 		if (day_end >= event_end) {
 			day_end = event_end;
@@ -635,9 +685,20 @@ process_multi_day (ECalConduitContext *ctxt, ECalChange *ccc, GList **multi_comp
 		dt_end.value = &end_value;
 		e_cal_component_set_dtend (clone, &dt_end);
 
+		e_cal_component_commit_sequence (clone);
+		
 		/* FIXME Error handling */
-		e_cal_create_object (ctxt->client, e_cal_component_get_icalcomponent (clone), NULL, NULL);
+		ical_comp = e_cal_component_get_icalcomponent (clone);
+		if (!ical_comp) {
+			ret = FALSE;
+			g_free (new_uid);
+			g_object_unref (clone);
+			goto cleanup;
+		}
 
+		e_cal_create_object (ctxt->client, ical_comp, NULL, NULL);
+
+		c = g_new0 (ECalChange, 1);
 		c->comp = clone;
 		c->type = E_CAL_CHANGE_ADDED;
 		
@@ -646,13 +707,19 @@ process_multi_day (ECalConduitContext *ctxt, ECalChange *ccc, GList **multi_comp
 
 		event_start = day_end;
 		day_end = time_day_end_with_zone (event_start, ctxt->timezone);
+
+		g_free (new_uid);
 	}
 	dt_start.value = old_start_value;
 	dt_end.value = old_end_value;
 	
 	e_cal_component_get_uid (ccc->comp, &uid);
 	/* FIXME Error handling */
-	e_cal_remove_object (ctxt->client, uid, NULL);
+	if (e_cal_component_is_instance (ccc->comp) || e_cal_component_has_recurrences (ccc->comp))
+		e_cal_remove_object_with_mod (ctxt->client, uid, NULL, CALOBJ_MOD_ALL, NULL);
+	else
+		e_cal_remove_object (ctxt->client, uid, NULL);
+
 	ccc->type = E_CAL_CHANGE_DELETED;
 
  cleanup:
@@ -1355,22 +1422,16 @@ pre_sync (GnomePilotConduit *conduit,
 	ctxt->dbi = dbi;	
 	ctxt->client = NULL;
 
-	if (start_calendar_server (ctxt) != 0) {
-		WARN(_("Could not start evolution-data-server"));
-		gnome_pilot_conduit_error (conduit, _("Could not start evolution-data-server"));
-		return -1;
-	}
-
 	/* Get the timezone */
 	ctxt->timezone = get_default_timezone ();
 	if (ctxt->timezone == NULL)
 		return -1;
 	LOG (g_message ( "  Using timezone: %s", icaltimezone_get_tzid (ctxt->timezone) ));
 	
-	/* Set the default timezone on the backend. */
-	if (ctxt->timezone) {
-		if (!e_cal_set_default_timezone (ctxt->client, ctxt->timezone, NULL))
-			return -1;
+	if (start_calendar_server (ctxt) != 0) {
+		WARN(_("Could not start evolution-data-server"));
+		gnome_pilot_conduit_error (conduit, _("Could not start evolution-data-server"));
+		return -1;
 	}
 
 	/* Get the default component */
@@ -1747,7 +1808,10 @@ delete_record (GnomePilotConduitSyncAbs *conduit,
 
 	e_pilot_map_remove_by_uid (ctxt->map, uid);
 	/* FIXME Error handling */
-	e_cal_remove_object (ctxt->client, uid, NULL);
+	if (e_cal_component_is_instance (local->comp) || e_cal_component_has_recurrences (local->comp))
+		e_cal_remove_object_with_mod (ctxt->client, uid, NULL, CALOBJ_MOD_ALL, NULL);
+	else
+		e_cal_remove_object (ctxt->client, uid, NULL);
 	
         return 0;
 }
